/* 
 * PPRacer 
 * Copyright (C) 2004-2005 Volker Stroebel <volker@planetpenguin.de>
 *
 * Copyright (C) 1999-2001 Jasmin F. Patry
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef AUDIO_H
#define AUDIO_H

#include "../../ppracer.h"

void init_audio();

bool is_audio_open();

void bind_sounds_to_context( CONST84 char *sound_context, CONST84 char **names, int num_sounds );
bool play_sound( const char *sound_context, int loop );
bool halt_sound( const char *sound_context );
bool set_sound_volume( const char *sound_context, int volume );
    

void bind_music_to_context( CONST84 char *music_context, CONST84 char *name, int loop );
bool play_music( char *music_context );
bool is_music_playing();
    
void update_audio();

void shutdown_audio();

void register_sound_tcl_callbacks( Tcl_Interp *ip );

#endif /* AUDIO_H */
